using System;
using System.Drawing;
using System.Windows.Forms;
using Dapfor.Net.Data;
using Dapfor.Net.Ui;

namespace Dapfor.Net.PerfDemo
{
    public partial class MainForm : Form
    {
        readonly Random _random = new Random();
        readonly ThreadSafeBindingList<TestClass> _bindingList = new ThreadSafeBindingList<TestClass>();
        readonly Timer _timer = new Timer();
        private DateTime _lastTick = DateTime.MinValue;
        private long _processedUpdates;

        public MainForm()
        {
            InitializeComponent();

            //Attach a data source
            _bindingList.AllowNew = false;
            grid.DataSource = _bindingList;
            grid.Highlighting.Enabled = false;

            //Prevent from default highlighting. 
            //We'll highlight cells with semi-transparent colors at each notification from TestClass objects
            grid.Highlighting.Fading = false;
            grid.Highlighting.Enabled = false;

            //Start timer
            _timer.Tick += OnTimerTick;
            _timer.Interval = 10;
            _timer.Start();

            //Populate collection with first 1000 items
            btnPopulate_Click(this, EventArgs.Empty);
        }

        void OnTimerTick(object sender, EventArgs e)
        {
            if(_bindingList.Count > 0)
            {
                //Update multiple objects at each timer tick
                int multiplier = trackBar.Value;
                for (int i = 0; i < multiplier; ++i)
                {
                    _processedUpdates++;
                    TestClass testClass = _bindingList[_random.Next(_bindingList.Count)];
                    testClass.UpdateRandomValue();
                }
            }

            //Print the statistics each second
            if ((DateTime.UtcNow - _lastTick) > TimeSpan.FromSeconds(1))
            {
                lbVisibleSize.Text = string.Format("Grid visible size: {0}x{1}", grid.ClientRectangle.Width, grid.ClientRectangle.Height);
                lbTotalUpdates.Text = string.Format("Total updates/sec: {0}", _processedUpdates);

                Row topVisibleRow = grid.Rows.TopVisibleRow;
                Row bottomVisibleRow = grid.Rows.BottomVisibleRow;
                long visibleUpdates = 0;
                if (topVisibleRow != null && bottomVisibleRow != null && grid.Rows.Count > 0)
                {
                    visibleUpdates = _processedUpdates*(bottomVisibleRow.VisibleIndex - topVisibleRow.VisibleIndex)/grid.Rows.Count;
                }
                lbVisibleUpdates.Text = string.Format("Visible updates/sec: {0}", visibleUpdates);


                //Reset stat counters
                _lastTick = DateTime.UtcNow;
                _processedUpdates = 0;
            }
        }

        private void btnPopulate_Click(object sender, EventArgs e)
        {
            for(int i = 0; i < 1000; ++i)
            {
                _bindingList.Add(new TestClass());
            }
        }

        private void grid_RowUpdated(object sender, GridRowUpdateEventArgs e)
        {
            //This method is called when the grid gets a notification from INotifyPropertyChanged

            //Highlight cells only if they are in the visible area of the grid
           // if(e.Row.IsInVisibleBounds)
            {
                //Get a value from the field, specified in the GridRowUpdateEventArgs
                TestClass tc = (TestClass)e.Row.DataObject;
                int value = 0;
                switch (e.DataField.Id)
                {
                    case "Value1": value = tc.Value1; break;
                    case "Value2": value = tc.Value2; break;
                    case "Value3": value = tc.Value3; break;
                    case "Value4": value = tc.Value4; break;
                    case "Value5": value = tc.Value5; break;
                    case "Value6": value = tc.Value6; break;
                    case "Value7": value = tc.Value7; break;
                    case "Value8": value = tc.Value8; break;
                    case "Value9": value = tc.Value9; break;
                    case "Value10": value = tc.Value10; break;
                }

                //Select a semi-transparent color
                Color color = value % 2 == 0 ? Color.Green : Color.Red;
                color = Color.FromArgb(100, color);
            
                //Highlight call with the selected color
                e.Row[e.DataField.Id].Highlight(TimeSpan.FromSeconds(2), color);
            }
        }

        private void cbFilterRows_CheckedChanged(object sender, EventArgs e)
        {
            if(cbFilterRows.Checked)
            {
                //Create an easy filter
                grid.Filter = new Filter(delegate(Row row)
                {
                    TestClass testClass = (TestClass)row.DataObject;
                    bool filtered = testClass.Value1 > 800000 ||
                                    testClass.Value2 > 800000 ||
                                    testClass.Value3 > 800000 ||
                                    testClass.Value4 > 800000 ||
                                    testClass.Value5 > 800000 ||
                                    testClass.Value6 > 800000 ||
                                    testClass.Value7 > 800000 ||
                                    testClass.Value8 > 800000 ||
                                    testClass.Value9 > 800000 ||
                                    testClass.Value10 > 800000;
                    return filtered;
                });
            }
            else
            {
                //Remove existing filter
                grid.Filter = null;
            }
        }
    }
}